@@ -40,7 +40,6 @@ proc jdoc:init {} {
 #   possibly jumping to a particular point in it
 #   the contents of the doc file will set up the Sections menu
 ##############################################################################
-### USE OF MOSAIC SHOULD BE A PREFERENCE!
 
 proc jdoc:load_topic { topic } {
   global JDOC_PREFS JDOC_PATH
@@ -51,13 +50,11 @@ proc jdoc:load_topic { topic } {
   set resource [lindex $tmp_list 0]
   set anchor [lindex $tmp_list 1]
   
-  # if it's a URL, view it with NCSA Mosaic
-  #   (THIS SHOULD BE A PREFERENCE!)
   if {[string match "http:*" $resource] || \
     [string match "ftp:*" $resource] || \
     [string match "*.html" $resource]} {
     wm withdraw .
-    exec Mosaic $resource
+    j:view_url $resource
     exit 0
   }
   
@@ -188,6 +185,24 @@ proc jdoc:configure_text { { t .t } } {
   $t tag configure jdoc:xref:link -underline 1
   $t tag bind jdoc:xref:manpage <ButtonRelease-1> {jdoc:x_manpage %W %x %y}
   
+  # list styles:
+  $t tag configure special:list_item:1 -font \
+    -*-symbol-*-*-*-*-*-120-*-*-*-*-*-*
+  $t tag configure special:list_item:2 -font \
+    -*-symbol-*-*-*-*-*-120-*-*-*-*-*-*
+  $t tag configure special:hr:normal -font nil2 -background black
+  catch {
+    $t tag configure list:level:1 -lmargin1 9m -lmargin2 9m
+    $t tag configure list:level:2 -lmargin1 18m -lmargin2 18m
+  }
+  catch {
+    $t tag configure special:list_item:1 -lmargin1 5m -lmargin2 5m
+    $t tag configure special:list_item:2 -lmargin1 14m -lmargin2 14m
+  }
+  $t tag raise special:list_item:1
+  $t tag raise special:list_item:2
+  $t tag raise special:hr:normal
+  
   # other configuration
   catch {.t configure -width $JDOC_PREFS(textwidth)}
   catch {.t configure -height $JDOC_PREFS(textheight)}
