@@ -40,6 +40,7 @@ j:ldb:set_defaults {
   {JEprompt:mode {Editing Mode:}}
   {JEconfirm:close_window {Are you sure you want to close this window?}}
   {JEconfirm:quit {Are you sure you want to quit?}}
+  {JEprompt:load {Load:}}
   {JEprompt:saveas {Save as:}}
   {JEconfirm:...exists_replace {File "$filename" exists; replace it?}}
   {JEconfirm:print_to... 
@@ -117,8 +118,13 @@ proc jedit:cmd:ask_mode { t args } {
 j:command:register jedit:cmd:close {Close Window}
 proc jedit:cmd:close { t args } {
   global JEDIT_WINDOW_COUNT
+  global JEDIT_FILES
+  global JEDIT_EMBEDDED
+  global JEDIT_COMMAND
   
-  if {$JEDIT_WINDOW_COUNT == 1} {
+  set toplevel [jedit:text_to_top $t]
+  
+  if {(!$JEDIT_EMBEDDED) && $JEDIT_WINDOW_COUNT == 1} {
     jedit:cmd:quit $t
   } else {
     set mode [jedit:get_mode $t]
@@ -135,6 +141,16 @@ proc jedit:cmd:close { t args } {
       mode:$mode:close $t
     }
   }
+  # thanks to ndanner@indiana.edu for the following:
+  # We need to unset this variable so that if we open a new window, jedit
+  # doesn't think that it already has a filename associated to it and hence
+  # overwrite that file when we try to save.  See jedit:get_filename .
+  if {[info exists JEDIT_FILES($toplevel)]} {
+    unset JEDIT_FILES($toplevel)
+  }
+  if {"x$JEDIT_COMMAND" != "x"} {
+    eval $JEDIT_COMMAND
+  }
 }
 
 ######################################################################
@@ -143,6 +159,11 @@ proc jedit:cmd:close { t args } {
 
 j:command:register jedit:cmd:quit {Quit}
 proc jedit:cmd:quit { t args } {
+  global JEDIT_EMBEDDED
+  if $JEDIT_EMBEDDED {
+    return
+  }
+  
   set mode [jedit:get_mode $t]
   
   if {[info procs mode:$mode:pre_quit_hook] != {}} {
@@ -166,7 +187,7 @@ j:command:register jedit:cmd:load {Load...}
 proc jedit:cmd:load { t args } {
   jedit:save_checkpoint $t	;# save undo information
 
-  set filename [j:fs -prompt "Load:"]
+  set filename [j:fs -prompt JEprompt:load]
   if {"x$filename" != "x"} then {
     jedit:set_filename $t $filename
     
@@ -252,8 +273,8 @@ proc jedit:cmd:insfile { t args } {
   jedit:save_checkpoint $t			;# save undo information
   set prompt_result [j:fs -prompt JEprompt:insert]
   if {$prompt_result != {}} then {
-    j:text:insert_string $t [exec cat $prompt_result]
-    j:text:insert_string $t "\n"
+    j:text:insert $t [j:fileio:read $prompt_result]
+    j:text:insert $t "\n"
   }
 }
 
@@ -263,12 +284,9 @@ proc jedit:cmd:insfile { t args } {
 
 j:command:register jedit:cmd:cut {Cut}
 proc jedit:cmd:cut { t args } {
-  global CUTBUFFER
-
   jedit:save_checkpoint $t			;# save undo information
-
-  set CUTBUFFER [$t get sel.first sel.last]
-  j:text:delete $t sel.first sel.last
+  
+  j:cmd:text_cut $t
 }
 
 ######################################################################
@@ -277,9 +295,7 @@ proc jedit:cmd:cut { t args } {
 
 j:command:register jedit:cmd:copy {Copy}
 proc jedit:cmd:copy { t args } {
-  global CUTBUFFER
-
-  set CUTBUFFER [$t get sel.first sel.last]
+  j:cmd:text_copy $t
 }
 
 ######################################################################
@@ -298,7 +314,7 @@ proc jedit:cmd:paste { t args } {
     mode:$mode:pre_paste_hook $t
   }
 
-  j:text:insert_string $t $CUTBUFFER
+  j:cmd:text_paste $t
 
   if {[info procs mode:$mode:post_paste_hook] != {}} {
     mode:$mode:post_paste_hook $t
@@ -353,7 +369,7 @@ proc jedit:cmd:run_command { t args } {
     if {$result != {}} {
       append result "\n"
       jedit:save_checkpoint $t			;# save undo information
-      j:text:insert_string $t $result
+      j:text:insert $t $result
     }
   }
 }
@@ -581,7 +597,7 @@ proc jedit:cmd:xpaste { t args } {
     mode:$mode:pre_xpaste_hook $t
   }
 
-  j:text:insert_string $t [j:selection_if_any]
+  j:text:insert $t [j:selection_if_any]
 
   if {[info procs mode:$mode:post_xpaste_hook] != {}} {
     mode:$mode:post_xpaste_hook $t
@@ -608,6 +624,24 @@ proc jedit:cmd:find_again { t args } {
   j:find:again $t
 }
 
+######################################################################
+# replace found string with replaced string
+# UTTERLY BOGUS - uses loads of information internal to j:find
+######################################################################
+
+j:command:register jedit:cmd:replace {Replace}
+proc jedit:cmd:replace { t args } {
+  global j_find
+  
+  if {! ([info exists j_find(replacewith)] && [j:text:has_selection $t])}  {
+    jedit:cmd:find
+  } else {
+    jedit:save_checkpoint $t
+    
+    j:text:replace $t sel.first sel.last $j_find(replacewith)
+  }
+}
+
 ######################################################################
 # hacks for more-specific kinds of finds (for vi/emacs bindings)
 ### BOGUS!  jedit should not need to know about the internals of j:find!
@@ -705,7 +739,7 @@ proc jedit:cmd:char_panel { t args } {
   pack $tl.b -anchor e
   
   bind $tl.t <ButtonRelease-1> "
-    j:text:insert_string $t \[%W get @%x,%y\]
+    j:text:insert $t \[%W get @%x,%y\]
   "
   foreach event {
     <ButtonRelease-3> <B3-Motion> <Button-3> <ButtonRelease-2>
@@ -728,7 +762,7 @@ proc jedit:cmd:char_panel { t args } {
 
 j:command:register jedit:cmd:hyphen {Insert Hyphen}
 proc jedit:cmd:hyphen { t args } {
-  j:text:insert_string $t "\xad"
+  j:text:insert $t "\xad"
 }
 
 ######################################################################
@@ -737,79 +771,7 @@ proc jedit:cmd:hyphen { t args } {
 
 j:command:register jedit:cmd:copyright {Insert Copyright Symbol}
 proc jedit:cmd:copyright { t args } {
-  j:text:insert_string $t "\xa9"
-}
-
-######################################################################
-# rich-text cut
-######################################################################
-
-j:command:register jedit:cmd:rich_cut {Rich Cut}
-proc jedit:cmd:rich_cut { t } {
-  jedit:cmd:rich_copy $t			;# (saves checkpoint)
-  $t delete sel.first sel.last
-}
-
-######################################################################
-# rich-text copy
-######################################################################
-
-j:command:register jedit:cmd:rich_copy {Rich Copy}
-proc jedit:cmd:rich_copy { t } {
-  global RICHBUFFER
-  
-  jedit:save_checkpoint $t			;# save undo information
-  
-  set RICHBUFFER {}
-  set curstring {}
-  set curtags [$t tag names sel.first]
-  
-  $t mark set richptr sel.first
-  
-  while {[$t compare richptr < sel.last]} {
-    set tags [$t tag names richptr]
-    set char [$t get richptr]
-    if {"x$tags" != "x$curtags"} {	;# new "range" of text
-      lappend RICHBUFFER [list $curstring $curtags]
-      set curstring $char
-      set curtags $tags
-    } else {
-      append curstring $char
-    }
-    $t mark set richptr {richptr+1c}
-  }
-  lappend RICHBUFFER [list $curstring $curtags]
-  return
-}
-
-######################################################################
-# rich-text paste
-#   partly lifted from insertWithTags in mkStyles.tcl demo
-######################################################################
-
-j:command:register jedit:cmd:rich_paste {Rich Paste}
-proc jedit:cmd:rich_paste { t } {
-  global RICHBUFFER
-  
-  jedit:save_checkpoint $t			;# save undo information
-  
-  lappend RICHBUFFER {}			;# make sure it's defined
-  
-  foreach pair $RICHBUFFER {
-    set text [lindex $pair 0]
-    set tags [lindex $pair 1]
-    
-    set start [$t index insert]
-    $t insert insert $text
-    foreach tag [$t tag names $start] {
-      $t tag remove $tag $start insert	;# clear tags inherited from left
-    }
-    foreach tag $tags {	
-      $t tag add $tag $start insert	;# add new tags
-    }
-  }
-  $t tag remove sel 1.0 end		;# clear selection (guaranteed in text)
-  return
+  j:text:insert $t "\xa9"
 }
 
 ######################################################################
@@ -889,6 +851,7 @@ j:command:register jedit:cmd:mode_prefs {Mode Preferences...}
 j:command:register jedit:cmd:save_checkpoint {Checkpoint}
 j:command:register jedit:cmd:undo {Undo}
 j:command:register jedit:cmd:redo {Redo}
+j:command:register jedit:font:plain {Plain Font}
 j:command:register jedit:font:roman {Roman}
 j:command:register jedit:font:italic {Italic}
 j:command:register jedit:font:bold {Bold}
@@ -900,7 +863,14 @@ j:command:register jedit:font:heading2 {Level 2 Heading}
 j:command:register jedit:font:heading3 {Level 3 Heading}
 j:command:register jedit:font:heading4 {Level 4 Heading}
 j:command:register jedit:font:heading5 {Level 5 Heading}
-j:command:register jedit:font:plain {Plain Font}
+j:command:register jedit:font:l_em {Emphasised Text}
+j:command:register jedit:font:l_cite {Citation}
+j:command:register jedit:font:l_var {Variable Reference}
+j:command:register jedit:font:l_dfn {Definition Text}
+j:command:register jedit:font:l_strong {Strong Text}
+j:command:register jedit:font:l_kbd {Keyboard}
+j:command:register jedit:font:l_code {Code Listing}
+j:command:register jedit:font:l_samp {Sample Text}
 j:command:register jedit:format:background:clear {Plain Background}
 j:command:register jedit:format:foreground:clear {Plain Foreground}
 
