@@ -21,7 +21,7 @@ j:ldb:set_defaults {
   {JEmenu:abbrev {Abbrev} 0}
   {JEmenu:filter {Filter} 2}
   {JEmenu:format {Format} 3}
-  {JEmenu:font {Font} -1}
+  {JEmenu:font {Font} 3}
   {JEmenu:background {Background} -1}
   {JEmenu:foreground {Foreground} -1}
   {JEmenucmd:underline {Underline} 0}
@@ -56,7 +56,7 @@ proc jedit:mkmenus {mb t} {
   
   set mode [jedit:get_mode $t]
   
-  set all_menus {editor file edit prefs abbrev filter format mode1 mode2}
+  set all_menus {editor file edit prefs abbrev filter format font mode1 mode2}
   set displayed_menus {}
   
   foreach menu $all_menus {
@@ -118,21 +118,27 @@ proc jedit:mkmenu:editor {menu t} {
 
 proc jedit:mkmenu:file {menu t} {
   global JEDIT_MODEPREFS
+  global JEDIT_EMBEDDED
+  global JEDIT_EMBEDDED_FILECOMMANDS
   
   j:menu:menubutton $menu $menu.m JEmenu:file
   
-  j:menu:commands $menu.m $t {
-    jedit:cmd:load
-    jedit:cmd:save
-    jedit:cmd:saveas
-    jedit:cmd:print
-    -
-    jedit:cmd:insfile
-    -
-    jedit:cmd:new_window
-    jedit:cmd:done
-    jedit:cmd:close
-    jedit:cmd:quit
+  if $JEDIT_EMBEDDED {
+    j:menu:commands $menu.m $t $JEDIT_EMBEDDED_FILECOMMANDS
+  } else {
+    j:menu:commands $menu.m $t {
+      jedit:cmd:load
+      jedit:cmd:save
+      jedit:cmd:saveas
+      jedit:cmd:print
+      -
+      jedit:cmd:insfile
+      -
+      jedit:cmd:new_window
+      jedit:cmd:done
+      jedit:cmd:close
+      jedit:cmd:quit
+    }
   }
 }
 
@@ -151,6 +157,7 @@ proc jedit:mkmenu:edit {menu t} {
     -
     jedit:cmd:find
     jedit:cmd:find_again
+    jedit:cmd:replace
     -
     jedit:cmd:go_to_line
     jedit:cmd:current_line
@@ -243,12 +250,6 @@ proc jedit:mkmenu:format {menu t} {
   jedit:mkmenu:format:foreground $menu.m.foreground $t
   jedit:mkmenu:format:characters $menu.m.characters $t
   
-  j:menu:commands $menu.m $t {
-    jedit:cmd:rich_cut
-    jedit:cmd:rich_copy
-    jedit:cmd:rich_paste
-    -
-  }
   $menu.m add cascade \
     -label [j:ldb JEmenu:font] \
     -menu $menu.m.font
@@ -286,6 +287,8 @@ proc jedit:mkmenu:format:font {menu t} {
   menu $menu
   
   j:menu:commands $menu $t {
+    jedit:font:plain
+    -
     jedit:font:roman
     jedit:font:italic
     jedit:font:bold
@@ -299,7 +302,14 @@ proc jedit:mkmenu:format:font {menu t} {
     jedit:font:heading4
     jedit:font:heading5
     -
-    jedit:font:plain
+    jedit:font:l_em
+    jedit:font:l_cite
+    jedit:font:l_var
+    jedit:font:l_dfn
+    jedit:font:l_strong
+    jedit:font:l_code
+    jedit:font:l_samp
+    jedit:font:l_kbd
   }
 }
 
@@ -314,6 +324,7 @@ proc jedit:mkmenu:format:background {menu t} {
   }
   
   set colours {
+    #000000 #ffffff
     Black Grey25 Grey33 Grey50 Grey66 Grey75 White
     Red Green Blue Cyan Magenta Yellow
     LemonChiffon Gold SpringGreen
@@ -338,6 +349,7 @@ proc jedit:mkmenu:format:foreground { menu t } {
   }
   
   set colours {
+    #000000 #ffffff
     Black Grey25 Grey33 Grey50 Grey66 Grey75 White
     Red Green Blue Cyan Magenta Yellow
     LemonChiffon Gold SpringGreen
@@ -361,6 +373,38 @@ proc jedit:mkmenu:format:characters { menu t } {
   }
 }
 
+proc jedit:mkmenu:font {menu t} {
+  global JEDIT_MODEPREFS
+  
+  j:menu:menubutton $menu $menu.m JEmenu:font
+  
+  j:menu:commands $menu.m $t {
+    jedit:font:plain
+    -
+    jedit:font:roman
+    jedit:font:italic
+    jedit:font:bold
+    jedit:font:bolditalic
+    jedit:font:typewriter
+    -
+    jedit:font:heading0
+    jedit:font:heading1
+    jedit:font:heading2
+    jedit:font:heading3
+    jedit:font:heading4
+    jedit:font:heading5
+    -
+    jedit:font:l_em
+    jedit:font:l_cite
+    jedit:font:l_var
+    jedit:font:l_dfn
+    jedit:font:l_strong
+    jedit:font:l_code
+    jedit:font:l_samp
+    jedit:font:l_kbd
+  }
+}
+
 proc jedit:mkmenu:mode1 { menu t } {
   set mode [jedit:get_mode $t]
   
@@ -405,7 +449,7 @@ proc jedit:prefs:menus { w mode } {
   global JEDIT_MODEPREFS
   
   frame $w
-  foreach menu {file edit prefs abbrev filter format mode1 mode2 user} {
+  foreach menu {file edit prefs abbrev filter format font mode1 mode2 user} {
     checkbutton $w.$menu -anchor w \
       -text [j:ldb JEpref:show...menu] \
       -variable JEDIT_MODEPREFS($mode,menu,$menu)
