@@ -204,7 +204,8 @@ proc jedit:cmd:mode_prefs { t args } {
         if {$JEDIT_MODEPREFS($tmp_mode,textheight) < 4} {
           set JEDIT_MODEPREFS($tmp_mode,textheight) 4
         }
-        jedit:apply_all_prefs %s
+        # noreread is a kludge!
+        jedit:apply_all_prefs %s noreread
         destroy .mode_prefs
       }
     }
