@@ -85,6 +85,8 @@ proc jedit:mkmain { w } {
 
 proc jedit:mkbuttonbar { w t } {
   global JEDIT_MODEPREFS
+  global JEDIT_EMBEDDED
+  global JEDIT_EMBEDDED_BUTTONS
   
   j:debug
   if [winfo exists $w] {
@@ -98,7 +100,11 @@ proc jedit:mkbuttonbar { w t } {
   if {"x[info procs mode:${mode}:mkbuttons]" != "x"} {
     mode:${mode}:mkbuttons $w.b $t
   } else {
-    j:command:buttonbar $w.b $t $JEDIT_MODEPREFS($mode,buttons)
+    if {$JEDIT_EMBEDDED && ("x$JEDIT_EMBEDDED_BUTTONS" != "x")} {
+      j:command:buttonbar $w.b $t $JEDIT_EMBEDDED_BUTTONS
+    } else {
+      j:command:buttonbar $w.b $t $JEDIT_MODEPREFS($mode,buttons)
+    }
   }
   pack $w.b [j:rule $w] -in $w -side top -fill x
   return $w
@@ -191,7 +197,7 @@ proc jedit:configure_text { t } {
   }
   j:configure_font $t $JEDIT_MODEPREFS($mode,textfont) ;# knows about `default'
   # for rich-text:
-  jedit:font:configure_fonts $t
+  j:tag:configure_fonts $t
 }
 
 ######################################################################
