@@ -1,7 +1,7 @@
 # jedit_util.tcl - utility procedures for jedit, a tk-based editor
 #
 ######################################################################
-# Copyright 1992-1995 by Jay Sekora.  This file may be freely        #
+# Copyright 1992-199d by Jay Sekora.  This file may be freely        #
 # distributed, modified or unmodified, for any purpose, provided     #
 # that this copyright notice is retained verbatim in all copies and  #
 # no attempt is made to obscure the authorship of this file.  If you #
@@ -52,18 +52,27 @@ proc jedit:init {} {
     {*.c		code		{C source}}
     {*.f		code		{Fortran source}}
     {*.h		code		{C header files}}
+    {*.xbm		code		{X bitmap (C code)}}
+    {*.xpm		code		{X pixmap (C code)}}
+    {*[mM]akefile*	code		{make(1) script}}
     {*.html		html		{WWW Hypertext Markup Language}}
     {*.htm		html		{WWW Hypertext Markup Language}}
+    {*.jhtml		jhtml		{WYSIWYG WWW Hypertext Markup Language}}
+    {*.jht		jhtml		{WYSIWYG WWW Hypertext Markup Language}}
     {*.jdoc		jdoc		{document for jdoc app.}}
     {*.jrt		richtext	{rich-text with fonts and other tags}}
     {*.p		code		{Pascal source}}
-    {*.sh		code		{Bourne shell script}}
-    {*.shar		code		{Bourne shell archive}}
+    {*.pl		code		{Perl or Prolog source}}
+    {*.ksh		sh		{Korn shell script}}
+    {*.sh		sh		{Bourne shell script}}
+    {*.shar		sh		{Bourne shell archive}}
     {*.tcl		tcl		{Tcl scripts}}
     {*.tk		tcl		{Tk/Tcl scripts}}
     {*.exp		tcl		{Expect scripts}}
     {*/.letter		mail		{Mail from tin}}
     {.letter		mail		{Mail from tin}}
+    {*/.followup	mail		{Posting from slrn}}
+    {.followup		mail		{Posting from slrn}}
     {*/.[a-zA-Z]*	code		{~/.login, etc.}}
     {.[a-zA-Z]*		code		{.login, etc.}}
     {*/draft*/[0-9]*	mh		{MH, exmh, xmh, etc.}}
@@ -76,6 +85,10 @@ proc jedit:init {} {
   					;# only first two sublist items matter
   set LINE_MODES {
     {%!*		code		{PostScript file}}
+    {{#!*/perl*}	code		{Perl scripts}}
+    {{#!*/sh*}		sh		{Bourne shell scripts}}
+    {{#!*/bash*}	sh		{bash scripts}}
+    {{#!*/ksh*}		sh		{Korn shell scripts}}
     {{#!*/wish*}	tcl		{Tk/Tcl scripts}}
     {{#!*/tclsh*}	tcl		{Tcl scripts}}
     {{#!*/expect*}	tcl		{Expect or Expectk scripts}}
@@ -139,11 +152,36 @@ proc jedit:jedit { args } {
   j:parse_args {
     {window unspecified}
     {mode default}
+    {embedded 0}
+    {command {}}
+    {filecommands {
+      jedit:cmd:load
+      jedit:cmd:save
+      jedit:cmd:saveas
+      jedit:cmd:print
+      -
+      jedit:cmd:insfile
+      -
+      jedit:cmd:close
+      jedit:cmd:done
+    }}
+    {buttons {}}
     {file {}}
     {line {}}
   }
   
   jedit:init				;# ignored second etc. time it's called
+  global JEDIT_EMBEDDED
+  global JEDIT_EMBEDDED_FILECOMMANDS
+  global JEDIT_EMBEDDED_BUTTONS
+  global JEDIT_COMMAND
+  set JEDIT_EMBEDDED $embedded		;# 1 -> disable exit, etc.
+  set JEDIT_COMMAND $command		;# called on window close
+  if $JEDIT_EMBEDDED {
+    set JEDIT_EMBEDDED_FILECOMMANDS \
+      $filecommands			;# commands to include in File menu
+    set JEDIT_EMBEDDED_BUTTONS $buttons	;# buttons to display in buttonbar
+  }
   
   if {"x$mode" == "xdefault"} {		;# if caller hasn't specified mode
     if {"x$file" != "x"} {		;#   and we have a filename
@@ -236,11 +274,12 @@ proc jedit:userinit {window mode file} {
 ######################################################################
 # apply editor and mode preferences (initially or after they change)
 ######################################################################
+# noreread is a kludge!
 
-proc jedit:apply_all_prefs { window } {
+proc jedit:apply_all_prefs { window {noreread ""}} {
   global JEDIT_MODEPREFS		;# mode-specific prefs
   jedit:apply_prefs $window
-  jedit:apply_mode $window
+  jedit:apply_mode $window $noreread
 }
 
 ######################################################################
