@@ -41,20 +41,25 @@ proc j:eb:basic_init { e } {
     
     {Delete			j:ekb:delete_left}
     {BackSpace			j:ekb:delete_left}
-    {Return			j:ekb:self_insert}
+    {Return			j:eb:no_op}
     
+    {Up				j:ekb:up}
+    {Down			j:ekb:down}
     {Left			j:ekb:left}
     {Right			j:ekb:right}
     
     {Home			j:ekb:bol}
     {End			j:ekb:eol}
     
-    {Control-v			j:eb:paste_selection}
+    {Next			j:ekb:scroll_down}
+    {Prior			j:ekb:scroll_up}
+    
+    {Control-v			j:ekb:paste_selection}
     {Control-i			j:ekb:self_insert}
     {Control-j			j:ekb:self_insert}
     {Control-h			j:ekb:delete_left}
     
-    {Control-DEFAULT		j:eb:no_op}
+    {Control-DEFAULT		j:eb:beep}
     {DEFAULT			j:ekb:self_insert}
     {Shift-DEFAULT		j:ekb:self_insert}
   }
