@@ -71,7 +71,7 @@ proc j:eb:edt_init { e } {
     {KP_Decimal			j:eb:beep}
     {KP_Decimal_X		j:ekb:edt:sel-range}
     
-    {Return			j:ekb:insert_newline}
+    {Return			j:eb:no_op}
     {BackSpace			j:ekb:delete_left}
     {Delete			j:ekb:delete_left}
     
@@ -205,10 +205,10 @@ proc j:ekb:edt:del-word { w k a } { j:eb:edt:delWord ekb $w $k $a }
 proc j:eb:edt:delWord { sfx w k a } {
   if [j:eb:edt:isForw $w] {
     ##### SHOULD RECORD WORD HERE
-    j:$sfx:delete_word_right $w $k $a
+    j:$sfx:delete_right_word $w $k $a
   } else {
     ##### SHOULD RECORD WORD HERE
-    j:$sfx:delete_word_left $w $k $a
+    j:$sfx:delete_left_word $w $k $a
   }
 }
 
