@@ -62,7 +62,7 @@ proc j:eb:vi_init { e } {
     {x				j:ekb:delete_right}
     {X				j:ekb:delete_left}
     {b				j:ekb:word_left}
-    {B				j:ekb:word_left}
+    {B				j:ekb:word_right}
     {e_XXX			j:ekb:vi:word_end}
     {w				j:ekb:word_right}
     {W				j:ekb:word_right}
@@ -94,12 +94,13 @@ proc j:eb:vi_init { e } {
     
     {Delete			j:ekb:delete_left}
     {BackSpace			j:ekb:delete_left}
+    {Return			j:eb:no_op}
     
     {Control-i			j:ekb:self_insert}
     {Control-j			j:ekb:self_insert}
     {Control-h			j:ekb:delete_left}
     
-    {Control-w			j:ekb:delete_word_left}
+    {Control-w			j:ekb:delete_left_word}
     {Control-u			j:ekb:vi:delete_left_line}
     
     {Escape			j:ekb:vi:command}
