@@ -27,7 +27,11 @@
 ######################################################################
 
 proc j:find {args} {
-  j:parse_args {{replace 1}}
+  j:parse_args {
+    {replace 1}
+    {findhistory j_find}
+    {replacehistory j_repl}
+  }
   
   set old_focus [focus]			;# so we can restore original focus
   set t $args				;# text widget to search in
@@ -35,6 +39,9 @@ proc j:find {args} {
   set j_find(widget) $t
   append j_find(searchfor) {}		;# make sure it's defined
   append j_find(replacewith) {}		;# make sure it's defined
+  
+  set j_find(findhistory) $findhistory
+  set j_find(replacehistory) $replacehistory
 
   if {! [info exists j_find(backwards)]} {
     set j_find(backwards) 0
@@ -54,13 +61,17 @@ proc j:find {args} {
   }
 
   toplevel .find
+  wm withdraw .find
+  
   wm title .find [j:ldb title:find_panel "Find Panel"]
   frame .find.t
   j:variable_entry .find.t.search \
     -label [j:ldb find:searchfor "Search for:"] \
+    -history $findhistory \
     -variable j_find(searchfor)
   j:variable_entry .find.t.replace \
     -label [j:ldb find:replacewith "Replace with:"] \
+    -history $replacehistory \
     -variable j_find(replacewith)
   frame .find.t.options
   label .find.t.options.filler -text {} -width 16 -anchor e
@@ -76,14 +87,20 @@ proc j:find {args} {
   set buttons(search) {
     search Search
       {
+        if ![string match "" $j_find(findhistory)] {
+          j:history:append $j_find(findhistory) $j_find(searchfor)
+        }
         if {[j:find:find_pattern $j_find(searchfor) $j_find(widget)] == 0} {
-          j:alert -text "Not found."
+          j:alert -text {Not found.}
         }
       }
     }
   set buttons(replace) {
     replace Replace
       {
+        if ![string match "" $j_find(replacehistory)] {
+          j:history:append $j_find(replacehistory) $j_find(replacewith)
+        }
         j:find:replace $j_find(searchfor) $j_find(replacewith) $j_find(widget)
         j:find:find_pattern $j_find(searchfor) $j_find(widget)
       }
@@ -91,6 +108,12 @@ proc j:find {args} {
   set buttons(replace_all) {
     replace_all {Replace All}
       {
+        if ![string match "" $j_find(findhistory)] {
+          j:history:append $j_find(findhistory) $j_find(searchfor)
+        }
+        if ![string match "" $j_find(replacehistory)] {
+          j:history:append $j_find(replacehistory) $j_find(replacewith)
+        }
         j:find:replace_all $j_find(searchfor) $j_find(replacewith) \
           $j_find(widget)
       }
@@ -138,12 +161,20 @@ proc j:find {args} {
     {.find.b.search invoke}
   bind .find.t.replace.e <Meta-g> \
     {.find.b.search invoke}
-
+  
+  # Meta-r in either field replaces:
+  bind .find.t.search.e <Meta-r> \
+    {.find.b.replace invoke}
+  bind .find.t.replace.e <Meta-r> \
+    {.find.b.replace invoke}
+  
   j:default_button .find.b.search .find.t.search.e .find.t.replace.e
   j:cancel_button .find.b.cancel .find.t.search.e .find.t.replace.e
 
   focus .find.t.search.e
   
+  update idletasks
+  wm deiconify .find
   tkwait window .find
   j:tk3 {focus $old_focus}
   j:tk4 {
