@@ -157,13 +157,6 @@ proc j:color_chooser \
 
 ######################################################################
 # j:variable_entry w ?options? - labelled entry for global variable
-# options include
-#   -label (default "Value:")
-#   -variable (default value - not really optional)
-#   -labelwidth (default 16)
-#   -entrywidth (default 40)
-#   -labelconfig (default "")
-#   -entryconfig (default "")
 # labelconfig and entryconfig are lists of additional configuration
 #   options to control the appearance of the label and the entry
 # BOGUSNESS:  you need to know that the entry is $newframe.e
@@ -178,9 +171,13 @@ proc j:variable_entry {args} {
     {entrywidth {40} }
     {labelconfig {} }
     {entryconfig {} }
+    {history {} }
   }
+  global jstools_library
   global $variable
-  
+
+  set doing_history ![string match "" $history]
+
   set label [uplevel 1 [list j:ldb $label]]
   
   set newframe [lindex $args 0]
@@ -190,7 +187,6 @@ proc j:variable_entry {args} {
   set e $newframe.e
   label $l -text $label -width $labelwidth -anchor e
   entry $e -width $entrywidth -textvariable $variable
-  
   if {$labelconfig != ""} {
     eval $l configure $labelconfig
   }
@@ -200,8 +196,43 @@ proc j:variable_entry {args} {
   pack $l -in $newframe -side left -fill both
   pack $e -in $newframe -side left -fill both
   
+  if $doing_history {
+    j:history:begin $history
+    
+    update idletasks
+    set h [winfo reqheight $newframe]
+    set ht 0
+    j:tk4 {
+      set ht [$e cget -highlightthickness]
+    }
+    set h [expr {$h - ( $ht * 2 ) - 6}]	;# um, derived empirically. :-) ######
+    button $newframe.down -width $h -height $h -padx 0 -pady 0 \
+      -bitmap @$jstools_library/bitmaps/down.xbm -command "
+      $e delete 0 end
+      $e insert end \[j:history:down $history\]
+      $e xview end
+    "
+    button $newframe.up -width $h -height $h -padx 0 -pady 0 \
+      -bitmap @$jstools_library/bitmaps/up.xbm -command "
+      $e delete 0 end
+      $e insert end \[j:history:up $history\]
+      $e xview end
+    "
+    pack $newframe.down $newframe.up -side left
+  
+    bind $e <Up> "
+      %W delete 0 end
+      %W insert end \[j:history:up $history\]
+      %W xview end
+      catch {break}
+    "
+    bind $e <Down> "
+      %W delete 0 end
+      %W insert end \[j:history:down $history\]
+      %W xview end
+      catch {break}
+    "
+  }
+  
   return $newframe
 }
-
-
-
