@@ -1,7 +1,7 @@
 # jprompts.tcl - various panels to ask the user for something
 #
 ######################################################################
-# Copyright 1992-1995 by Jay Sekora.  This file may be freely        #
+# Copyright 1992-1996 by Jay Sekora.  This file may be freely        #
 # distributed, modified or unmodified, for any purpose, provided     #
 # that this copyright notice is retained verbatim in all copies and  #
 # no attempt is made to obscure the authorship of this file.  If you #
@@ -27,12 +27,6 @@ j:default J_PREFS(confirm) 1
 
 ######################################################################
 # j:prompt ?options? - prompt the user for information
-# options are:
-#   -text (default "Enter a value:"
-#   -default (default "")
-#   -cancelvalue (default "")
-#   -file (default 0)
-#   -title (default "Prompt")
 # if $file, then the Tab key will do filename completion
 ######################################################################
 
@@ -42,46 +36,81 @@ proc j:prompt { args } {
     {default ""}
     {cancelvalue ""}
     {file 0}
+    {show {}}
     {title title:prompt}
+    {history ""}
   }
   
   set text [uplevel 1 [list j:ldb $text]]
   set title [uplevel 1 [list j:ldb $title]]
   
   global j_prompt
-
+  global jstools_library
+  
+  set doing_history ![string match "" $history]
+  
   set old_focus [focus]			;# so we can restore original focus
 
   toplevel .pr
+  wm withdraw .pr			;# lets us "update" without flashing
+  					;# will be undone by j:dialogue
   wm title .pr $title
   
   message .pr.msg -width 300 -anchor w -text $text
-  entry .pr.field -width 40
+  
+  frame .pr.mid				;# has entry, history buttons if any
+  entry .pr.mid.e -width 40
+  pack .pr.mid.e -in .pr.mid -side left -fill x
+  if $doing_history {
+    update idletasks
+    set h [winfo reqheight .pr.mid]
+    set ht 0
+    j:tk4 {
+      set ht [.pr.mid.e cget -highlightthickness]
+    }
+    set h [expr {$h - ( $ht * 2 ) - 6}]	;# um, derived empirically. :-) ######
+    button .pr.mid.down -width $h -height $h -padx 0 -pady 0 \
+      -bitmap @$jstools_library/bitmaps/down.xbm -command "
+      .pr.mid.e delete 0 end
+      .pr.mid.e insert end \[j:history:down $history\]
+      .pr.mid.e xview end
+    "
+    button .pr.mid.up -width $h -height $h -padx 0 -pady 0 \
+      -bitmap @$jstools_library/bitmaps/up.xbm -command "
+      .pr.mid.e delete 0 end
+      .pr.mid.e insert end \[j:history:up $history\]
+      .pr.mid.e xview end
+    "
+    pack .pr.mid.down .pr.mid.up -side left
+  }
+  catch {.pr.mid.e configure -show $show} ;# ### NEEDS ERROR CHECKING (tk3) ###
+  
   j:buttonbar .pr.b -default ok -buttons [format {
-    {ok OK {set j_prompt(result) [.pr.field get]; destroy .pr}}
+    {ok OK {set j_prompt(result) [.pr.mid.e get]; destroy .pr}}
+    {clear Clear {.pr.mid.e delete 0 end}}
     {cancel Cancel {set j_prompt(result) {%s}; destroy .pr}}
   } $cancelvalue]
 
   pack .pr.msg -side top -fill both -expand yes -padx 10
-  pack .pr.field -side top -padx 10 -pady 10
+  pack .pr.mid -side top -padx 10 -pady 10
   pack .pr.b -side bottom -fill x
   pack [j:rule .pr -width 200] -side bottom -fill x
 
-  .pr.field delete 0 end
-  .pr.field insert end $default
+  .pr.mid.e delete 0 end
+  .pr.mid.e insert end $default
   
   j:dialogue .pr			;# position in centre of screen
 
   if $file {
     j:tk3 {
-      bind .pr.field <Tab> {
+      bind .pr.mid.e <Tab> {
         set f [%W get]
         %W delete 0 end
         %W insert end [j:expand_filename $f]
       }
     }
     j:tk4 {
-      bind .pr.field <Tab> {
+      bind .pr.mid.e <Tab> {
         set f [%W get]
         %W delete 0 end
         %W insert end [j:expand_filename $f]
@@ -91,15 +120,36 @@ proc j:prompt { args } {
       }
     }
   }
-  j:default_button .pr.b.ok .pr.field
-  j:cancel_button .pr.b.cancel .pr.field
+  j:default_button .pr.b.ok .pr.mid.e
+  j:cancel_button .pr.b.cancel .pr.mid.e
+  
+  if $doing_history {
+    j:history:begin $history
+    bind .pr.mid.e <Up> "
+      %W delete 0 end
+      %W insert end \[j:history:up $history\]
+      %W xview end
+      catch {break}
+    "
+    bind .pr.mid.e <Down> "
+      %W delete 0 end
+      %W insert end \[j:history:down $history\]
+      %W xview end
+      catch {break}
+    "
+  }
 
-  focus .pr.field
+  focus .pr.mid.e
   update
   grab .pr
   tkwait window .pr
   j:tk3 {focus $old_focus}
   j:tk4 {focus -force $old_focus}	;# can't figure out a better way...
+  if $doing_history {
+    if {"x$j_prompt(result)" != "x$cancelvalue"} {
+      j:history:append $history $j_prompt(result)
+    }
+  }
   return $j_prompt(result)
 }
 
@@ -125,7 +175,10 @@ proc j:prompt_font { args } {
 # j:prompt_tcl - prompt for a tcl command and execute it
 ######################################################################
 
-proc j:prompt_tcl {} {
+proc j:prompt_tcl { args } {
+  j:parse_args {
+    {history j_tcl}
+  }
   global j_prompt_tcl
   append j_prompt_tcl(RESULT) {}
   
@@ -134,7 +187,7 @@ proc j:prompt_tcl {} {
   set title [j:ldb title:tcl:result "Result of Tcl command"]
   
   set prompt_result [j:prompt \
-    -text $prompt -default $j_prompt_tcl(RESULT)]
+    -history $history -text $prompt]
   if {$prompt_result != {}} then {
     set j_prompt_tcl(RESULT) $prompt_result
     set result [uplevel #0 $j_prompt_tcl(RESULT)]
@@ -156,7 +209,10 @@ proc j:prompt_tcl {} {
 # j:prompt_unix - prompt for a unix command and execute it
 ######################################################################
 
-proc j:prompt_unix {} {
+proc j:prompt_unix { args } {
+  j:parse_args {
+    {history j_unix}
+  }
   global j_prompt_unix
   append j_prompt_unix(RESULT) {}
   
@@ -164,7 +220,7 @@ proc j:prompt_unix {} {
   set title_short [j:ldb title:unix:result_short {Command output}]
   
   set prompt_result [j:prompt \
-    -text $prompt -default $j_prompt_unix(RESULT)]
+    -history $history -text $prompt]
   if {$prompt_result != {}} then {
     set j_prompt_unix(RESULT) $prompt_result
     set command $prompt_result
@@ -201,6 +257,7 @@ proc j:prompt_colour_name { args } {
   j:parse_args {
     {prompt prompt:colour_name}
     {title title:colour_name}
+    {default magenta}
   }
   
   set prompt [uplevel 1 [list j:ldb $prompt]]
@@ -214,14 +271,6 @@ proc j:prompt_colour_name { args } {
   
   set old_focus [focus]			;# so we can restore original focus
   
-###   if {[info exists j_prompt(count)]} then {
-###     set j_prompt(count) [expr {$j_prompt(count) + 1}]
-###   } else {
-###     set j_prompt(count) 0
-###   }
-### 
-###   set w ".prompt$j_prompt(count)"
-
   set w ".prompt_colour_name"
   toplevel $w
   wm title $w $title
@@ -249,7 +298,7 @@ proc j:prompt_colour_name { args } {
     ] \
   ]
   frame $w.frame -width 100 -height 100 \
-    -background bisque -relief raised -borderwidth 2
+    -background $default -relief raised -borderwidth 2
   frame $w.list
   scrollbar $w.list.sb -command "$w.list.lb yview"
   listbox $w.list.lb -yscroll "$w.list.sb set" -relief flat -setgrid true
@@ -310,7 +359,7 @@ proc j:prompt_colour_name { args } {
   tkwait window $w
   j:tk3 {focus $old_focus}
   j:tk4 {focus -force $old_focus}	;# can't figure out a better way...
-  if {$j_prompt(colour) == ""} {set j_prompt(colour) bisque}
+  if {$j_prompt(colour) == ""} {set j_prompt(colour) $default}
   return $j_prompt(colour)
 }
 
@@ -333,16 +382,22 @@ proc j:prompt_colour_rgb { args } {
   j:parse_args {
     {prompt prompt:colour_rgb}
     {title title:colour_rgb}
+    {default {#ff80ff}}
   }
   
+  set hexbyte {[0-9a-fA-F][0-9a-fA-F]}
+  
+  if ![string match #$hexbyte$hexbyte$hexbyte $default] {
+    error "illegal RGB colour $default"
+  }
   set prompt [uplevel 1 [list j:ldb $prompt]]
   set title [uplevel 1 [list j:ldb $title]]
   
   global j_prompt
   
-  set j_prompt(red) 255
-  set j_prompt(blue) 196
-  set j_prompt(green) 228
+  # extract hex values of default/current colour as decimal numbers
+  scan $default "#%2x%2x%2x" j_prompt(red) j_prompt(green) j_prompt(blue)
+  
   set j_prompt(flag) 0
   
   set old_focus [focus]			;# so we can restore original focus
@@ -360,9 +415,12 @@ proc j:prompt_colour_rgb { args } {
   bind $w.value <1> {%W select from @0; %W select to end}
   frame $w.scales
   set j_prompt(flag) 1
-  j:prompt_colour_rgb:make_scale $w $w.scales.red red 255 Red
-  j:prompt_colour_rgb:make_scale $w $w.scales.green green 255 Green
-  j:prompt_colour_rgb:make_scale $w $w.scales.blue blue 255 Blue
+  j:prompt_colour_rgb:make_scale $w $w.scales.red red 255 \
+    [j:ldb colour_name:red Red]
+  j:prompt_colour_rgb:make_scale $w $w.scales.green green 255 \
+    [j:ldb colour_name:green Green]
+  j:prompt_colour_rgb:make_scale $w $w.scales.blue blue 255 \
+    [j:ldb colour_name:blue Blue]
   pack $w.scales.red $w.scales.green $w.scales.blue \
     -side left -fill y -expand yes
   set j_prompt(flag) 0
@@ -389,7 +447,7 @@ proc j:prompt_colour_rgb { args } {
   
   j:prompt_colour_rgb:update_colour $w red $j_prompt(red)
   
-  j:default_button $w.b.ok $w
+  j:default_button $w.b.ok $w $w.value
   focus $w
   j:dialogue $w
   tkwait window $w
@@ -404,6 +462,9 @@ proc j:prompt_colour_rgb:make_scale {w name var to title} {
   frame $name
   scale $name.scale -to $to \
     -command "j:prompt_colour_rgb:update_colour $w $var"
+  if {$to >= 100} {
+    $name.scale configure -length $to
+  }
   $name.scale set [set j_prompt($var)]
   label $name.label -text $title
   pack $name.label -in $name
