@@ -1,7 +1,7 @@
 # jtagconvert.tcl - procedures for converting multi-font text
 #
 ######################################################################
-# Copyright 1992-1995 by Jay Sekora.  This file may be freely        #
+# Copyright 1992-1996 by Jay Sekora.  This file may be freely        #
 # distributed, modified or unmodified, for any purpose, provided     #
 # that this copyright notice is retained verbatim in all copies and  #
 # no attempt is made to obscure the authorship of this file.  If you #
@@ -47,6 +47,18 @@ j:default J_TAGCONVERT(html,jdoc_prefix) ""
 j:default J_TAGCONVERT(tclrt,prologue) prologue.tclrt
 j:default J_TAGCONVERT(tclrt,epilogue) epilogue.tclrt
 
+# following should be in J_TAGCONVERT
+global J_PREFS
+j:default J_PREFS(ps_roman_font) /Times-Roman
+j:default J_PREFS(ps_italic_font) /Times-Italic
+j:default J_PREFS(ps_bold_font) /Times-Bold
+j:default J_PREFS(ps_bolditalic_font) /Times-BoldItalic
+j:default J_PREFS(ps_monospace_font) /Courier
+j:default J_PREFS(ps_heading_font) /Helvetica
+j:default J_PREFS(ps_monospace_size) 11
+j:default J_PREFS(ps_normal_size) 12
+j:default J_PREFS(ps_linespacing) 14
+
 ######################################################################
 # j:tc:saveas t -
 #   let the user choose a file format and a file to save the
@@ -100,7 +112,9 @@ proc j:tc:mark_transitions { t } {
   foreach tag [$t tag names] {
     # we only care about particular kinds of tags at this point:
     if {[string match richtext:font:* $tag] || \
-        [string match jdoc:link:* $tag]} {
+        [string match jdoc:link:* $tag] || \
+        [string match list:level:* $tag] || \
+        [string match special:* $tag]} {
       foreach place [$t tag ranges $tag] {
         $t mark set convert:place:$place $place
       }
@@ -183,9 +197,19 @@ proc j:tc:generic_convert_text { t {mode tclrt} args } {
   }
   
   set result {}
-  
+  set title [j:tag:get_metainfo $t title]
+  if [string match "" $title] {
+    set title Untitled
+    catch {
+      set title \
+        [$t get richtext:font:heading0.first richtext:font:heading0.last]
+      set title [lindex [split $title "\n"] 0]
+      set title [string trim $title]
+    }
+  }
+
   catch {
-    set result [j:tc:$mode:prologue $t]
+    set result [j:tc:$mode:prologue $t -title $title]
   }
   
   j:tc:clear_marks $t
@@ -207,7 +231,9 @@ proc j:tc:generic_convert_text { t {mode tclrt} args } {
     set end [lindex $marks [expr $i + 1]]
     set text [$t get $start $end]
     set tags [$t tag names $start]
-    append result [j:tc:$mode:convert_chunk $text $tags]
+    set nextchar [$t get $end $end]
+    set nexttags [$t tag names $end]
+    append result [j:tc:$mode:convert_chunk $text $tags $nextchar $nexttags]
   }
   catch {
     append result [j:tc:$mode:epilogue $t]
@@ -308,14 +334,14 @@ proc j:tc:tclrt:convert_text { t } {
   return [j:tc:generic_convert_text $t tclrt -break 1]
 }
 
-proc j:tc:tclrt:prologue { t } {
+proc j:tc:tclrt:prologue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
     $J_TAGCONVERT(lib)/$J_TAGCONVERT(tclrt,prologue)]
 }
 
-proc j:tc:tclrt:epilogue { t } {
+proc j:tc:tclrt:epilogue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
@@ -340,11 +366,19 @@ proc j:tc:tclrt:tag_to_command { tag } {
     *:heading3		{return j:rt:h3}
     *:heading4		{return j:rt:h4}
     *:heading5		{return j:rt:h5}
+    *:l_em		{return j:rt:it}
+    *:l_cite		{return j:rt:it}
+    *:l_var		{return j:rt:it}
+    *:l_dfn		{return j:rt:it}
+    *:l_strong		{return j:rt:bf}
+    *:l_kbd		{return j:rt:bf}
+    *:l_code		{return j:rt:tt}
+    *:l_samp		{return j:rt:tt}
     *			{return j:rt:tt}
   }
 }
 
-proc j:tc:tclrt:convert_chunk { text tags } {
+proc j:tc:tclrt:convert_chunk { text tags nextchar nexttags } {
   set text [j:tc:tclrt:convert_string $text]
   set tag [j:tc:find_font_tag $tags]
   set command [j:tc:tclrt:tag_to_command $tag]
@@ -358,18 +392,18 @@ proc j:tc:tclrt:convert_chunk { text tags } {
 ######################################################################
 ######################################################################
 
-proc j:tc:tex:convert_text { t } {
+proc j:tc:tex:convert_text { t args } {
   return [j:tc:generic_convert_text $t tex -break 1]
 }
 
-proc j:tc:tex:prologue { t } {
+proc j:tc:tex:prologue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
     $J_TAGCONVERT(lib)/$J_TAGCONVERT(tex,prologue)]
 }
 
-proc j:tc:tex:epilogue { t } {
+proc j:tc:tex:epilogue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
@@ -406,11 +440,19 @@ proc j:tc:tex:tag_to_command { tag } {
     *:heading3		{return \\jtcheadingthree}
     *:heading4		{return \\jtcheadingfour}
     *:heading5		{return \\jtcheadingfive}
+    *:l_em		{return \\jtcitalic}
+    *:l_cite		{return \\jtcitalic}
+    *:l_var		{return \\jtcitalic}
+    *:l_dfn		{return \\jtcitalic}
+    *:l_strong		{return \\jtcbold}
+    *:l_kbd		{return \\jtcbold}
+    *:l_code		{return \\jtctypewriter}
+    *:l_samp		{return \\jtctypewriter}
     *			{return \\jtctypewriter}
   }
 }
 
-proc j:tc:tex:convert_chunk { text tags } {
+proc j:tc:tex:convert_chunk { text tags nextchar nexttags } {
   set text [j:tc:tex:convert_string $text]
   set tag [j:tc:find_font_tag $tags]
   set command [j:tc:tex:tag_to_command $tag]
@@ -428,19 +470,36 @@ proc j:tc:tex:convert_chunk { text tags } {
 ######################################################################
 ######################################################################
 
-proc j:tc:html:convert_text { t } {
+proc j:tc:html:convert_text { t args } {
   return [j:tc:generic_convert_text $t html -break 0]
 }
 
 
-proc j:tc:html:prologue { t } {
+proc j:tc:html:prologue { t args } {
+  j:parse_args {
+    {title Untitled}
+  }
+  
   global J_TAGCONVERT
   
-  return [j:tc:get_file_contents \
+  set prologue [j:tc:get_file_contents \
     $J_TAGCONVERT(lib)/$J_TAGCONVERT(html,prologue)]
+  regsub -all @@TITLE@@ $prologue $title prologue
+  
+  set bodyparams {}
+  foreach attribute {background bgcolor text link vlink alink} {
+    set value {}
+    catch {set value [j:tag:get_metainfo $t body_$attribute]}
+    if ![string match {} $value] {
+      append bodyparams " $attribute=\"$value\""
+    }
+  }
+  
+  regsub -all -nocase "<body>" $prologue "<body$bodyparams>" prologue
+  return $prologue
 }
 
-proc j:tc:html:epilogue { t } {
+proc j:tc:html:epilogue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
@@ -471,7 +530,10 @@ proc j:tc:html:convert_string { text } {
     } ;# foreach
   } ;# if any <>&
   
-  regsub -all -- "\n\n*" $text "\n<p>\n" text
+  # change newlines to breaks:
+  regsub -all -- "\n" $text "\n<br>\n" text
+  # change _multiple_ breaks to paragraphs:
+  regsub -all -- "\n<br>\n\n<br>\n(\n<br>\n)*" $text "\n<p>\n" text
   return [j:tc:break_string $text]
 }
 
@@ -481,23 +543,87 @@ proc j:tc:html:tag_to_font { tag } {
     *:italic		{return i}
     *:bold		{return b}
     *:bolditalic	{return em}
-    *:typewriter	{return samp}
+    *:typewriter	{return tt}
     *:heading0		{return h1}
     *:heading1		{return h2}
     *:heading2		{return h3}
     *:heading3		{return h4}
     *:heading4		{return h5}
     *:heading5		{return h5}
-    *			{return samp}
+    *:l_em		{return em}
+    *:l_cite		{return cite}
+    *:l_var		{return var}
+    *:l_dfn		{return dfn}
+    *:l_strong		{return strong}
+    *:l_kbd		{return kbd}
+    *:l_code		{return code}
+    *:l_samp		{return samp}
+    *			{return tt}
   }
 }
 
-proc j:tc:html:convert_chunk { text tags } {
+proc j:tc:html:convert_chunk { text tags nextchar nexttags } {
   global J_TAGCONVERT
-  set text [j:tc:html:convert_string $text]
+  
+  # hack for <ul>...</ul> - I need a more general way to deal with this.
+  global J_TAGCONVERT_HTML
+  j:default J_TAGCONVERT_HTML(list_level) 0
+  
+  # hack for vertical whitespace - again, this needs to be more general
+  j:default J_TAGCONVERT_HTML(last_whitespace) none
+  
   set before ""
   set after ""
   set link ""
+  
+  # eliminate (at least some) superfluous <br> and <p>:
+  if {[string length [j:tc:find_tag {*:heading*} $nexttags]] ||
+      [string length [j:tc:find_tag {special:*} $nexttags]] ||
+      [string length [j:tc:find_tag {*:heading*} $tags]]} {
+    set text [string trimright $text "\n \t"]
+    set after "\n"			;# make HTML easier to read
+  }
+  
+  set specialtag [j:tc:find_tag "special:*" $tags]
+  if ![llength $specialtag] {
+    set text [j:tc:html:convert_string $text]
+  } else {
+    switch -glob $specialtag {
+      "special:list_item:*" {
+        set text "\n<li>"
+      }
+      "special:hr:*" {
+        set text "\n<hr>\n"
+        set J_TAGCONVERT_HTML(last_whitespace) hr
+      }
+      "special:html:literal" {
+        set text $text
+      }
+    }
+  }
+  
+  # check for transition to or from a list:
+  set listtag [j:tc:find_tag "list:level:*" $tags]
+  if [string match "" $listtag] {
+    set current_list_level 0
+  } else {
+    set current_list_level [lindex [split $listtag :] 2]
+  }
+  set old_list_level $J_TAGCONVERT_HTML(list_level)
+  if {$current_list_level > $old_list_level} {
+    set count [expr $current_list_level - $old_list_level]
+    for {set i 0} {$i < $count} {incr i} {
+      set before "$before\n<ul>\n"		;# spit out <ul>
+    }
+  }
+  if {$current_list_level < $old_list_level} {
+    set count [expr $old_list_level - $current_list_level]
+    for {set i 0} {$i < $count} {incr i} {
+      set before "\n</ul>\n$before"		;# spit out </ul>
+    }
+  }
+  set J_TAGCONVERT_HTML(list_level) $current_list_level
+  
   set linktag [j:tc:find_tag "jdoc:link:*" $tags]
   if [string length $linktag] {
     set link [string range $linktag 10 end]	;# strip off "jdoc:link:"
@@ -538,18 +664,44 @@ proc j:tc:html:convert_chunk { text tags } {
 ######################################################################
 ######################################################################
 
-proc j:tc:ps:convert_text { t } {
+proc j:tc:ps:convert_text { t args } {
   return [j:tc:generic_convert_text $t ps -break 0]
 }
 
-proc j:tc:ps:prologue { t } {
-  global J_TAGCONVERT
+proc j:tc:ps:prologue { t args } {
+  global J_TAGCONVERT J_PREFS
   
-  return [j:tc:get_file_contents \
+  set prologue "%!"
+  append prologue "% FONTS:\n"
+  append prologue "/romanfont {$J_PREFS(ps_roman_font)} def\n"
+  append prologue "/italicfont {$J_PREFS(ps_italic_font)} def\n"
+  append prologue "/boldfont {$J_PREFS(ps_bold_font)} def\n"
+  append prologue "/bolditalicfont {$J_PREFS(ps_bolditalic_font)} def\n"
+  append prologue "/typewriterfont {$J_PREFS(ps_monospace_font)} def\n"
+  append prologue "/normalsize $J_PREFS(ps_normal_size) def\n"
+  append prologue {
+    /romansize normalsize def
+    /italicsize normalsize def
+    /boldsize normalsize def
+    /bolditalicsize normalsize def
+  }
+  append prologue "/typewritersize $J_PREFS(ps_monospace_size) def\n"
+  append prologue "/headingfont {$J_PREFS(ps_heading_font)} def\n"
+  append prologue {
+    /heading0font headingfont def
+    /heading1font headingfont def
+    /heading2font headingfont def
+    /heading3font headingfont def
+    /heading4font headingfont def
+    /heading5font headingfont def
+  }
+  append prologue "/linespacing $J_PREFS(ps_linespacing) def\n"
+  append prologue [j:tc:get_file_contents \
     $J_TAGCONVERT(lib)/$J_TAGCONVERT(ps,prologue)]
+  return $prologue
 }
 
-proc j:tc:ps:epilogue { t } {
+proc j:tc:ps:epilogue { t args } {
   global J_TAGCONVERT
   
   return [j:tc:get_file_contents \
@@ -578,14 +730,72 @@ proc j:tc:ps:tag_to_command { tag } {
     *:heading3		{return heading3}
     *:heading4		{return heading4}
     *:heading5		{return heading5}
+    *:l_em		{return italic}
+    *:l_cite		{return italic}
+    *:l_var		{return italic}
+    *:l_dfn		{return italic}
+    *:l_strong		{return bold}
+    *:l_kbd		{return bold}
+    *:l_code		{return typewriter}
+    *:l_samp		{return typewriter}
     *			{return typewriter}
   }
 }
 
-proc j:tc:ps:convert_chunk { text tags } {
-  set text [j:tc:ps:convert_string $text]
+proc j:tc:ps:convert_chunk { text tags nextchar nexttags } {
+  global J_TAGCONVERT
+  
+  # hack for <ul>...</ul> - I need a more general way to deal with this.
+  global J_TAGCONVERT_PS
+  j:default J_TAGCONVERT_PS(list_level) 0
+  
+  
+  set before ""
+  
+  # check for transition to or from a list:
+  set listtag [j:tc:find_tag "list:level:*" $tags]
+  if [string match "" $listtag] {
+    set current_list_level 0
+  } else {
+    set current_list_level [lindex [split $listtag :] 2]
+  }
+  set old_list_level $J_TAGCONVERT_PS(list_level)
+  if {$current_list_level > $old_list_level} {
+    set count [expr $current_list_level - $old_list_level]
+    for {set i 0} {$i < $count} {incr i} {
+      append before "indent\n"		;# spit out "indent"
+    }
+  }
+  if {$current_list_level < $old_list_level} {
+    set count [expr $old_list_level - $current_list_level]
+    for {set i 0} {$i < $count} {incr i} {
+      append before "unindent\n"	;# spit out "unindent"
+    }
+  }
+  set J_TAGCONVERT_PS(list_level) $current_list_level
+  
+  
+  set specialtag [j:tc:find_tag "special:*" $tags]
+  if ![llength $specialtag] {
+    set text [j:tc:ps:convert_string $text]
+  } else {
+    switch -glob $specialtag {
+      "special:list_item:*" {
+        return "${before}item\n"
+      }
+      "special:hr:*" {
+        return "${before}hrule\n"
+        #set J_TAGCONVERT_HTML(last_whitespace) hr
+      }
+      "special:html:literal" {
+        return "${before}% there was some HTML literal text omitted here\n"
+      }
+    }
+  }
+  
+  
   set tag [j:tc:find_font_tag $tags]
   set command [j:tc:ps:tag_to_command $tag]
   
-  return "$command $text breakshow\n"
+  return "$before$command $text breakshow\n"
 }
