@@ -21,3 +21,33 @@ proc j:default { var value } {
   }
   return [uplevel 1 [list set $var]]
 }
+
+######################################################################
+# j:mkdir dir - try to make directory $dir (and parents if necessary)
+######################################################################
+
+proc j:mkdir { dir } {
+  global tcl_version
+  if ![info exists tcl_version] {
+    set tcl_version [info tclversion]
+  }
+  if {$tcl_version >= 7.6} {
+    file mkdir $dir			;# tcl 1.6 has this built-in:
+    return
+  }
+  
+  set dir [string trimright $dir /]	;# make sure doesn't end in /
+  
+  if [file isdirectory $dir] {
+    return
+  }
+  
+  set parent [file dirname $dir]
+  if [string length $parent] {
+    j:mkdir $parent
+  }
+  if [catch {exec mkdir $dir}] {
+    error "Couldn't create directory $dir"
+  }
+}
+  
\ No newline at end of file
