@@ -31,8 +31,7 @@ if {! [info exists J_PREFS(typeover)]} {set J_PREFS(typeover) 1}
 ### PROBLEM - checking J_PREFS(typeover) shouldn't really be here; it's
 ###   only appropriate if a user event is generating the text to insert
 
-proc j:text:insert_string { w text } {
-  j:debug
+proc j:text:insert { w text } {
   global j_text
   global j_tag
   global J_PREFS
@@ -60,15 +59,25 @@ proc j:text:insert_string { w text } {
   set j_text(dirty,$w) 1
 }
 
+proc j:text:insert_string { w text } [info body j:text:insert]
+
 ######################################################################
 # j:text:move w index -
 #   move insert mark in w to index
 ######################################################################
 
 proc j:text:move { w index } {
-  j:debug
-  $w mark set insert $index
-  $w yview -pickplace insert
+  global j_text
+  global j_tag
+  global J_PREFS
+  
+  # if we're using tagged-insertion...
+  if [info exists j_tag(tags,$w)] {		;# using tagged text
+    j:tag:move $w $index
+  } else {
+    $w mark set insert $index
+    $w yview -pickplace insert
+  }
 }
 
 ######################################################################
@@ -77,7 +86,6 @@ proc j:text:move { w index } {
 ######################################################################
 
 proc j:text:delete { w from to } {
-  j:debug
   j:text:move $w $from
   $w delete $from $to
   set j_text(dirty,$w) 1
@@ -190,3 +198,76 @@ proc j:text:insert_touches_selection { t } {
   return 1
 }
 
+######################################################################
+######################################################################
+######################################################################
+
+######################################################################
+# j:text:get_text_with_tags t index1 index2
+#   return a list, suitable for use with $t insert, of tagged text
+#   from $index1 to $index2
+######################################################################
+
+# NOTE - this requires Tk4.1 or later!  Under earlier versions it
+#   just returns the text and the tags on the first character.
+proc j:text:get_text_with_tags { t index1 index2 } {
+  catch {unset tags}
+  set tags(0) 1
+  unset tags(0)
+  
+  j:tkversion >= 4.1 {
+    foreach tag [$t tag names $index1] {
+      set tags($tag) 1
+    }
+    
+    set rvalue {}
+    
+    foreach {key value index} [$t dump -text -tag $index1 $index2] {
+      switch -exact $key {
+        tagon {
+          set tags($value) 1
+        }
+        tagoff {
+          catch {unset tags($value)}
+        }
+        text {
+          lappend rvalue $value
+          lappend rvalue [array names tags]
+        }
+      }
+    }
+  }
+  j:tkversion < 4.1 {
+    j:beep $t
+    set rvalue [list [$t get $index1 $index2] [$t tag names $index1]]
+  }
+  
+  return $rvalue
+}
+
+######################################################################
+# j:text:tagged_insert $t text [tag text2 tag2...]
+#   insert tagged text into $t, like text widget insert command
+######################################################################
+
+# NOTE - this requires Tk4.0 or later!
+proc j:text:tagged_insert { t tagged_text } {
+  global j_text
+  global j_tag
+  global J_PREFS
+  
+  j:tk3 {
+    error "Insufficient Tk version; requires at least 4.0"
+  }
+  
+  # in typeover mode, all insertions replace selection:
+  if $J_PREFS(typeover) {
+    if [j:text:insert_touches_selection $t] {	;# else might be off-screen!
+      j:text:delete $t sel.first sel.last
+    }
+  }
+  
+  eval [list $t] insert insert $tagged_text
+  $t see insert
+  set j_text(dirty,$t) 1
+}
