@@ -1,7 +1,7 @@
 # jtexttags.tcl - tagged typing and tagging the selection for Text bindings
 #
 ######################################################################
-# Copyright 1992-1995 by Jay Sekora.  This file may be freely        #
+# Copyright 1992-1996 by Jay Sekora.  This file may be freely        #
 # distributed, modified or unmodified, for any purpose, provided     #
 # that this copyright notice is retained verbatim in all copies and  #
 # no attempt is made to obscure the authorship of this file.  If you #
@@ -12,7 +12,7 @@
 
 # TO DO: clear_tags, clear_marks
 
-# see also j:text:insert_string in jtext.tcl
+# see also j:text:insert in jtext.tcl
 
 #
 # tags used by these functions are of the form $category:$attribute:$value,
@@ -47,6 +47,47 @@ proc j:tag:insert_string { w text } {
   }
 }
 
+######################################################################
+# move to $index in $w, setting current tags to tags prevalent there
+######################################################################
+
+proc j:tag:move { w index } {
+  global j_tag
+  
+  set j_tag(tags,$w) {}
+  
+  $w mark set insert $index
+  $w yview -pickplace insert
+  set tags_after [$w tag names insert]
+  set tags_before [$w tag names insert-1char]
+  if [$w compare $index == 1.0] {
+    # if we're at the beginning of the widget, pretend the preceding
+    #   character has the same tags
+    set tags_before $tags_after
+  }
+  
+  foreach tag $tags_before {
+    switch -glob $tag {
+      special:list_item:* -
+      special:hr:* {
+        # don't pick up list_item or horizontal rule tags
+      }
+      richtext:* -
+      display:* {
+        # pick up all ordinary font and display tags on preceding character
+        lappend j_tag(tags,$w) $tag
+      }
+      default {
+        # pick up other tags only when both preceding and following 
+        #   characters share them
+        if {[lsearch -exact $tags_after $tag] >= 0} {
+          lappend j_tag(tags,$w) $tag
+        }
+      }
+    }
+  }
+}
+
 ######################################################################
 # set tags to use when inserting typed text
 #   if $tags is the empty list, inserted text will be plain.
@@ -229,6 +270,39 @@ proc j:tag:configure_display_tags { w } {
   }
 }
 
+######################################################################
+# configure standard fonts per user's preferences
+######################################################################
+
+proc j:tag:configure_fonts { w } {
+  global J_PREFS
+  
+  foreach pair {
+    {roman screen_roman_font}
+    {italic screen_italic_font}
+    {bold screen_bold_font}
+    {bolditalic screen_bolditalic_font}
+    {typewriter screen_monospace_font}
+    {heading0 screen_heading0_font}
+    {heading1 screen_heading1_font}
+    {heading2 screen_heading2_font}
+    {heading3 screen_heading3_font}
+    {heading4 screen_heading4_font}
+    {heading5 screen_heading5_font}
+    {l_em screen_italic_font}
+    {l_cite screen_italic_font}
+    {l_var screen_italic_font}
+    {l_dfn screen_italic_font}
+    {l_strong screen_bold_font}
+    {l_kbd screen_bold_font}
+    {l_code screen_monospace_font}
+    {l_samp screen_monospace_font}
+  } {
+    set tag richtext:font:[lindex $pair 0]
+    set font $J_PREFS([lindex $pair 1])
+    j:configure_tag_font $w $tag $font}
+}
+
 ######################################################################
 # return non-text content (tags and marks) of a text widget
 ######################################################################
@@ -271,16 +345,98 @@ proc j:tag:set_annotation { {t} {state} } {
   $t yview -pickplace insert
 }
 
+######################################################################
+# get a piece of metainfo associated with a text widget by key
+# if no key is specified, return whole list of metainfo
+######################################################################
+
+proc j:tag:get_metainfo { args } {
+  j:parse_args {
+    {default {}}
+  }
+  
+  set t [lindex $args 0]
+  set key [lindex $args 1]		;# {} if not given
+  
+  global j_textinfo
+  
+  append j_textinfo($t) {}		;# make sure it exists
+  
+  if [string match "" $key] {
+    return $j_textinfo($t)
+  }
+  
+  foreach pair $j_textinfo($t) {
+    if {[string compare $key [lindex $pair 0]] == 0} {
+      return [lindex $pair 1]
+    }
+  }
+  return $default
+}
+
+######################################################################
+# set a piece of metainfo associated with a text widget by key
+######################################################################
+
+proc j:tag:set_metainfo { t key value} {
+  global j_textinfo
+  
+  append j_textinfo($t) {}		;# make sure it exists
+  
+  # start with a new list containing only the new key and value
+  set new_textinfo [list [list $key $value]]
+  
+  # now copy the rest of the list into the new list, skipping key that matches
+  foreach pair $j_textinfo($t) {
+    if {[string compare $key [lindex $pair 0]] != 0} {
+      lappend new_textinfo $pair
+    }
+  }
+  
+  # now replace the old list with the new one
+  set j_textinfo($t) $new_textinfo
+  return
+}
+
+######################################################################
+# delete a piece of metainfo associated with a text widget by key
+######################################################################
+
+proc j:tag:delete_metainfo { t key} {
+  global j_textinfo
+  
+  append j_textinfo($t) {}		;# make sure it exists
+  
+  # start with a new empty list
+  set new_textinfo {}
+  
+  # now copy the rest of the list into the new list, skipping key that matches
+  foreach pair $j_textinfo($t) {
+    if {[string compare $key [lindex $pair 0]] != 0} {
+      lappend new_textinfo $pair
+    }
+  }
+  
+  # now replace the old list with the new one
+  set j_textinfo($t) $new_textinfo
+  return
+}
+
 ######################################################################
 # archive text widget to file, including state
 ######################################################################
 
 proc j:tag:archive_text_widget { {t} {filename} } {
+  global j_textinfo
+  
+  if ![info exists j_textinfo($t)] {
+    set j_textinfo($t) {}
+  }
   j:tk3 {set text [$t get 1.0 end]}
   j:tk4 {set text [$t get 1.0 end-1c]}
   set state [j:tag:get_annotation $t]
-  
-  j:fileio:write $filename [list $text $state]
+  set metainfo $j_textinfo($t)
+  j:fileio:write $filename [list $text $state $metainfo]
 }
 
 ######################################################################
@@ -288,10 +444,13 @@ proc j:tag:archive_text_widget { {t} {filename} } {
 ######################################################################
 
 proc j:tag:restore_text_widget { {t} {filename} } {
+  global j_textinfo
+  
   set archive [j:fileio:read $filename]
   
   set text [lindex $archive 0]
   set state [lindex $archive 1]
+  set metainfo [lindex $archive 2]
   
   set state_option \
     [lindex [$t configure -state] 4]		;# in case it's disabled
@@ -299,9 +458,10 @@ proc j:tag:restore_text_widget { {t} {filename} } {
   
   $t delete 1.0 end
   $t insert end $text
-  j:tag:set_annotation $t $state
+  j:tag:set_annotation $t $state		;# set tags and marks
   
   $t configure -state $state_option		;# might have been disabled
+  set j_textinfo($t) $metainfo			;# set out-of-band metainfo
 }
 
 ######################################################################
@@ -375,4 +535,3 @@ proc j:tag:offset_index { index offset } {
     return [expr $index + $line_offset]
   }
 }
-
