@@ -1,7 +1,7 @@
 # jtkversion.tcl - procedures to deal with multiple Tk versions
 #
 ######################################################################
-# Copyright 1992-1995 by Jay Sekora.  This file may be freely        #
+# Copyright 1992-1996 by Jay Sekora.  This file may be freely        #
 # distributed, modified or unmodified, for any purpose, provided     #
 # that this copyright notice is retained verbatim in all copies and  #
 # no attempt is made to obscure the authorship of this file.  If you #
@@ -47,3 +47,35 @@ proc j:tk3 { {command ""} } {
     return 0
   }
 }
+
+######################################################################
+# j:tkversion 
+#  perform an action if running under a specific range of Tk versions
+######################################################################
+
+proc j:tkversion { op versions {command ""} } {
+  global tk_version
+  
+  if [string match "range" $op] {
+    set from [lindex $versions 0]
+    set to [lindex $versions 1]
+    
+    if {$tk_version >= $from && $tk_version <= $to} {
+      if [string length $command] {
+        uplevel 1 [list eval $command]
+      }
+      return 1
+    } else {
+      return 0
+    }
+  } else {
+    if [list $tk_version $op $versions] {
+      if [string length $command] {
+        uplevel 1 [list eval $command]
+      }
+      return 1
+    } else {
+      return 0
+    }
+  }
+}
\ No newline at end of file
